import unittest
from datetime import datetime
import pandas as pd
from pandas._testing import assert_frame_equal

from wsbtrading.utils import dates


class TestConvertUnixTimestampToDate(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['idx', 'timestamp', 'eventDate']
        data = [
            (0, 1593556445000, '2020-06-30'),
            (1, 1592951645000, '2020-06-23'),
            (2, 1592778845000, '2020-06-21'),
            (3, 1593641161682, '2020-07-01'),
            (4, 1583715763996, '2020-03-09'),
            (5, 1593640898412, '2020-07-01'),
        ]
        # yapf: enable
        self.expected_df_no_time = pd.DataFrame(data=data, columns=schema)

        schema = ['idx', 'timestamp', 'eventDateTime']
        data = [
            (0, 1593556445000, '2020-06-30 22:34:05'),
            (1, 1592951645000, '2020-06-23 22:34:05'),
            (2, 1592778845000, '2020-06-21 22:34:05'),
            (3, 1593641161682, '2020-07-01 22:06:01'),
            (4, 1583715763996, '2020-03-09 01:02:43'),
            (5, 1593640898412, '2020-07-01 22:01:38'),
        ]
        self.expected_df_with_time = pd.DataFrame(data=data, columns=schema)

    def test_convert_unix_timestamp_to_date_kernel(self):
        """Ensures the correct timestamp to date conversion"""
        # no time
        timestamp = 1593556445000
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        assert actual == '2020-06-30'

        timestamp = 1593642847059
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        assert actual == '2020-07-01'

        timestamp = 1593641161682
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        assert actual == '2020-07-01'

        timestamp = 1593640898412
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        assert actual == '2020-07-01'

        timestamp = 1583715763996
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        assert actual == '2020-03-09'

        timestamp = 1583715789701
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        assert actual == '2020-03-09'

        # with time
        timestamp = 1583715789701
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=True)
        assert actual == '2020-03-09 01:03:09'

        timestamp = 1583715763996
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=True)
        assert actual == '2020-03-09 01:02:43'

        timestamp = 1593640898412
        actual = dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=True)
        assert actual == '2020-07-01 22:01:38'

    def test_convert_unix_timestamp_to_date_kernel_udf(self):
        """Ensures the correct UDF implementation for unix_timestamp_to_date"""
        # no time
        actual = self.expected_df_no_time[['idx', 'timestamp']]
        actual['eventDateTime'] = actual['timestamp']\
            .apply(lambda x: dates.convert_unix_timestamp_to_date_kernel(timestamp=x, output_time=True))

        assert_frame_equal(actual, self.expected_df_with_time, check_dtype=True)

        # with time
        actual = self.expected_df_with_time[['idx', 'timestamp']]
        actual['eventDateTime'] = actual['timestamp']\
            .apply(lambda x: dates.convert_unix_timestamp_to_date_kernel(timestamp=x, output_time=True))

        assert_frame_equal(actual, self.expected_df_with_time, check_dtype=True)


class TestParseDateFromString(unittest.TestCase):
    def test_parse_date_from_string(self):
        """Proper date object is constructed."""
        actual = dates.parse_date_from_string('2020-04-20')
        expected = datetime.strptime('%s 00:00:00' % '2020-04-20', '%Y-%m-%d %H:%M:%S')

        assert actual == expected


class TestBuildDateRange(unittest.TestCase):
    def test_build_date_range(self):
        """Proper date range is constructed."""
        actual = dates.build_date_range(start_date='2020-04-01', end_date='2020-04-03')
        expected = ['2020-04-01', '2020-04-02', '2020-04-03']

        assert actual == expected

        # end of year
        actual = dates.build_date_range(start_date='2019-12-30', end_date='2020-01-01')
        expected = ['2019-12-30', '2019-12-31', '2020-01-01']

        assert actual == expected

        # leap year
        actual = dates.build_date_range(start_date='2020-02-28', end_date='2020-03-02')
        expected = ['2020-02-28', '2020-02-29', '2020-03-01', '2020-03-02']

        assert actual == expected


class TestDateRangeToUnixRange(unittest.TestCase):
    def test_date_range_to_unix_range(self):
        """Date range is converted to a unix range."""
        actual = dates.date_range_to_unix_range(date_range=['2020-04-01', '2020-04-02', '2020-04-03'])
        expected = '{2020-04-01,2020-04-02,2020-04-03}'

        assert actual == expected


class TestDateSlider(unittest.TestCase):
    def test_date_slider(self):
        """Checks sliding by a positive number of days."""
        actual = dates.date_slider(date='2020-01-01', days_to_add=7)
        expected = '2020-01-08'

        assert actual == expected

        # zero days
        actual = dates.date_slider(date='2020-01-01', days_to_add=0)
        expected = '2020-01-01'

        assert actual == expected

        # negative days
        actual = dates.date_slider(date='2020-01-01', days_to_add=-2)
        expected = '2019-12-30'

        assert actual == expected



import requests
import json
import alpaca_trade_api as tradeapi

from typing import Dict

from wsbtrading.instrumentation import Alpaca as iAlpaca


def get_account(trading_type: str) -> Dict[str, str]:
    """Returns a JSON blog of open order.

    Args:
        trading_type: denotes live versus paper trading

    Returns:
        a dictionary of account information, such as cash on hand, account value, etc.

    Sample:

    .. code-block:

        Account({'account_blocked': False,
                 'account_number': 'PA3CXEIHWXHP',
                 'buying_power': '3990803.201',
                 'cash': '1000000',
                 'created_at': '2021-01-30T16:30:05.404915Z',
                 'currency': 'USD',
                 'daytrade_count': 0,
                 'daytrading_buying_power': '3990803.201',
                 'equity': '1000000',
                 'id': 'f53f4501-58b1-4527-b565-8b1cf61dfb17',
                 'initial_margin': '4598.3995',
                 'last_equity': '1000000',
                 'last_maintenance_margin': '0',
                 'long_market_value': '0',
                 'maintenance_margin': '0',
                 'multiplier': '4',
                 'pattern_day_trader': False,
                 'portfolio_value': '1000000',
                 'regt_buying_power': '1990803.201',
                 'short_market_value': '0',
                 'shorting_enabled': True,
                 'sma': '0',
                 'status': 'ACTIVE',
                 'trade_suspended_by_user': False,
                 'trading_blocked': False,
                 'transfers_blocked': False})

    More info [here](https://alpaca.markets/docs/api-documentation/api-v2/account/)

    **Example**

    .. code-block:: python

        from wsbtrading.order import account
        account.get_account(trading_type='paper_trading')
    """
    base_url = iAlpaca.api_call[trading_type]['base_url']
    api_key = iAlpaca.api_key
    api_secret = iAlpaca.api_secret

    api = tradeapi.REST(api_key, api_secret, base_url, api_version='v2')

    return api.get_account()



__version__ = '1.0.0a12'

__doc__ = """wsbtrading is a library that handles data I/O, aggregation, and modeling to facilitate algorithmic trading stategies."""

import logging
import uuid
from typing import List


def load_logging_config():
    logging.basicConfig(format='%(asctime)s - %(name)s - [%(levelname)s] - %(message)s')


def generate_staging_version() -> str:
    return __version__ + '_' + uuid.uuid4().hex[:8]


class MissingColumnError(Exception):
    """Custom error raised when a dataframe is missing a column required for some calculation."""


def check_columns(dataframe: 'DataFrame', required_columns: List[str]):
    """Checks if the dataframe has all the columns.

    Raises:
        MissingColumnError: if any of the required_columns not in the dataframe's schema
    """
    missing_columns = [column for column in required_columns if column not in dataframe.columns.to_series()]
    if missing_columns:
        raise MissingColumnError(f'The dataframe requires the following columns: {required_columns}. '
                                 f'Missing columns are: {missing_columns}.')



from wsbtrading.data_io import snapshot_daily
from wsbtrading.maths import is_in_squeeze
from wsbtrading.order import order

dict_of_df = snapshot_daily.read_snapshot()
stock_ticker_list = dict_of_df.values()

# TODO: differentiate between a positive squeeze or negative squeeze to determine if buying or selling is a good idea
for stock_ticker in stock_ticker_list:
    df = dict_of_df[stock_ticker]
    is_in_squeeze = is_in_squeeze(df=df,
                                  metric_col='Close',
                                  low_col='Low',
                                  high_col='High',
                                  rolling_window=20)

    if is_in_squeeze:
        print('value will print money!')
        order.execute_order(symbol=stock_ticker,
                            qty=100,
                            side='buy',
                            type='market',
                            time_in_force='gtc',
                            trading_type='paper_trading')

    else:
        print('This stock is junk!')
        print('Not touching it with a 10-foot stick.')



import unittest
from unittest.mock import Mock, patch

import pytest

from wsbtrading.data_io import data_io


class TestAlpacaApiConnection(unittest.TestCase):
    @patch.object(data_io, 'alpaca_rest_api_conn')
    def test_alpaca_api_connection(self, AlpacaApiConnMock: Mock):
        alpaca_api_conn_mock = Mock()
        AlpacaApiConnMock.return_value = alpaca_api_conn_mock

        data_io.alpaca_rest_api_conn(trading_type='paper_trading')

        AlpacaApiConnMock.assert_called_with(trading_type='paper_trading')


class TestPostgresConnection(unittest.TestCase):
    @patch.object(data_io, 'postgres_conn')
    def test_postgres_conn(self, PostgresConnMock: Mock):
        postgres_conn_mock = Mock()
        PostgresConnMock.return_value = postgres_conn_mock

        data_io.postgres_conn()

        PostgresConnMock.assert_called_with()


class TestGeneratePathToWrite(unittest.TestCase):
    """Ensures path is built correctly."""
    def test_generate_path_to_write(self):
        # Unsupported lob
        with pytest.raises(ValueError):
            _ = data_io.generate_path_to_write(environment='prod',
                                               granularity='unsupported_granularity',
                                               dataset_name='test_dataset')

        # Writing to default base path
        actual = data_io.generate_path_to_write(environment='prod',
                                                granularity='daily',
                                                dataset_name='test_dataset',
                                                timestamp='2020-09-15_01-39-54')
        assert actual == '/group/wsbtrading/prod/daily/test_dataset/2020-09-15_01-39-54'

        # Write to another base path
        actual = data_io.generate_path_to_write(environment='prod',
                                                granularity='daily',
                                                dataset_name='share_prices',
                                                root_path='/user/bdeely/',
                                                timestamp='version_1')
        assert actual == '/user/bdeely/prod/daily/share_prices/version_1'

        # Extra slash at end of base path
        actual = data_io.generate_path_to_write(environment='prod',
                                                granularity='daily',
                                                dataset_name='share_prices',
                                                root_path='/user/bdeely/',
                                                timestamp='version_1')
        assert actual == '/user/bdeely/prod/daily/share_prices/version_1'

        # Spaces and caps in project name
        actual = data_io.generate_path_to_write(environment='prod',
                                                granularity='daily',
                                                dataset_name='My DaTASET Name',
                                                timestamp='job_run_v4')
        assert actual == '/group/wsbtrading/prod/daily/my_dataset_name/job_run_v4'



import pathlib

#import finrl

import pandas as pd
import datetime
import os
#pd.options.display.max_rows = 10
#pd.options.display.max_columns = 10


#PACKAGE_ROOT = pathlib.Path(finrl.__file__).resolve().parent
#PACKAGE_ROOT = pathlib.Path().resolve().parent

#TRAINED_MODEL_DIR = PACKAGE_ROOT / "trained_models"
#DATASET_DIR = PACKAGE_ROOT / "data"

# data
#TRAINING_DATA_FILE = "data/ETF_SPY_2009_2020.csv"
TRAINING_DATA_FILE = "data/dow_30_2009_2020.csv"

now = datetime.datetime.now()
TRAINED_MODEL_DIR = f"trained_models/{now}"
os.makedirs(TRAINED_MODEL_DIR)
TURBULENCE_DATA = "data/dow30_turbulence_index.csv"

TESTING_DATA_FILE = "test.csv"





import requests
import json
from typing import Dict, Any

from wsbtrading.instrumentation import Alpaca as iAlpaca


def execute_order(stock_ticker: str, qty: int, side: str, type: str, time_in_force: str, trading_type: str) \
        -> Dict[str, Any]:
    """Configures and executes an order.

    Args:
        stock_ticker: the company's stock ticker
        qty: the number of shares to purchase
        side: allows you to choose 'buy', 'sell' side (i.e. buying shares or selling them)
        type: possible values for ``type`` are 'market', 'limit', 'stop', 'stop_limit', 'trailing_stop'
        time_in_force: possible values are 'day', 'gtc', 'opg', 'cls', 'ioc', 'fok'. For more info check
        trading_type: denotes live versus paper trading
        [here](https://alpaca.markets/docs/trading-on-alpaca/orders/#time-in-force)

    Returns:
        a float representing one number that was divided by another number

    **Example**

    .. code-block:: python

        from wsbtrading.order import order
        order.execute_order(
            stock_ticker='TSLA',
            qty=10,
            side='buy',
            type='market',
            time_in_force='gtc',
            trading_type='paper_trading'
        )
    """
    order_url = iAlpaca.api_call[trading_type]['sub_urls']['order_url']
    headers = iAlpaca.headers
    # TODO: add support for more complex order types (https://alpaca.markets/docs/trading-on-alpaca/orders/#bracket-orders)
    data = {
        "symbol": stock_ticker,
        "qty": qty,
        "side": side,
        "type": type,
        "time_in_force": time_in_force
    }

    r = requests.post(url=order_url, json=data, headers=headers)

    return json.loads(r.content)


def get_orders(trading_type: str) -> Dict[str, Any]:
    """Returns a JSON blog of open order(s).

    Args:
        trading_type: denotes live versus paper trading

    Returns:
    .. code-block:

        [{'id': 'acd7cfe6-086b-4ff7-a10d-3eb7ec84b3c7', 'client_order_id': '8e77087b-b548-4a19-a81b-7ad030beeb3d',
        'created_at': '2021-01-23T17:21:02.916741Z', 'updated_at': '2021-01-23T17:21:02.916741Z',
        'submitted_at': '2021-01-23T17:21:02.90977Z', 'filled_at': None, 'expired_at': None, 'canceled_at': None,
        'failed_at': None, 'replaced_at': None, 'replaced_by': None, 'replaces': None,
        'asset_id': 'b0b6dd9d-8b9b-48a9-ba46-b9d54906e415', 'symbol': 'AAPL', 'asset_class': 'us_equity',
        'qty': '100', 'filled_qty': '0', 'filled_avg_price': None, 'order_class': '', 'order_type': 'market',
        'type': 'market', 'side': 'buy', 'time_in_force': 'gtc', 'limit_price': None, 'stop_price': None,
        'status': 'accepted', 'extended_hours': False, 'legs': None, 'trail_percent': None, 'trail_price': None,
        'hwm': None}]

    More info [here](https://alpaca.markets/docs/api-documentation/api-v2/orders/)

    **Example**

    .. code-block:: python

        from wsbtrading.order import order
        order.get_orders(trading_type='paper_trading')
    """
    order_url = iAlpaca.api_call[trading_type]['sub_urls']['order_url']
    headers = iAlpaca.headers

    r = requests.get(order_url, headers=headers)

    return json.loads(r.content)



"""Functions to wrangle dates."""

from datetime import datetime, timedelta
from typing import List


def convert_unix_timestamp_to_date_kernel(timestamp: int, output_time: bool = False) -> str:
    """Converts an integer timestamp into date string.

    Args:
        timestamp: the timestamp (in ms) to convert to date string
        output_time: a flag whether to include time in the conversion

    Returns:
        date string extracted out of the timestamp

    **Example**

    .. code-block:: python

        from wsbtrading.utils import dates
        timestamp = 1609459200
        dates.convert_unix_timestamp_to_date_kernel(timestamp=timestamp, output_time=False)
        # '2020-07-01'
        df_mapped = df.withColumn(
            'eventDate',
            dates.convert_unix_timestamp_to_date_kernel.udf(
                col('timestamp'), lit(True)
            )
        )
    """
    return datetime.utcfromtimestamp(timestamp / 1000).strftime('%Y-%m-%d %H:%M:%S' if output_time else '%Y-%m-%d')


def parse_date_from_string(date: str) -> datetime:
    """Converts a date from a string to a datetime.

    Args:
        date: the date to parse from string representation to datetime

    Returns:
        datetime object of the date

    **Example**

    .. code-block:: python

        from wsbtrading.utils import dates
        dates.parse_date_from_string('2020-04-20')
    """
    return datetime.strptime(f'{date} 00:00:00', '%Y-%m-%d %H:%M:%S')


def build_date_range(start_date: str, end_date: str) -> List[str]:
    """Creates an array of dates, useful for creating time ranges for analysis.

    Args:
        start_date: the date to start with
        end_date: the date to end with

    Returns:
        list of datetime object filling the range of dates

    **Example**

    .. code-block:: python

        from wsbtrading.utils import dates
        dates.build_date_range(start_date='2020-04-01', end_date='2020-04-03')
        # ['2020-04-01', '2020-04-02', '2020-04-03']
    """
    start_date = parse_date_from_string(start_date)
    end_date = parse_date_from_string(end_date)

    return [(start_date + timedelta(days=+x)).strftime('%Y-%m-%d') for x in range(0, (end_date - start_date).days + 1)]


def date_range_to_unix_range(date_range: List[str]) -> str:
    """Formats an array of dates to a string that can be used in file paths names.

    Args:
        date_range: a list of dates, formatted as YYYY-MM-DD

    Returns:
        a string that looks like a dict filled with dates

    **Example**

    .. code-block:: python

        from wsbtrading.utils import dates
        dates.date_range_to_unix_range(['2020-04-01', '2020-04-02', '2020-04-03'])
        # '{2020-04-01,2020-04-02,2020-04-03}'
    """
    return '{%s}' % ','.join(date_range)


def date_slider(date: str, days_to_add: int) -> str:
    """Takes a date string and shifts it.

    Note:
        useful for times when you want to look back N-number of days, such as looking at time windows

    Args:
        date: the date to be shifted, formatted as YYYY-MM-DD
        days_to_add: the number of days to shift by

    Returns:
        the shifted date

    **Example**

    .. code-block:: python

        from wsbtrading.utils import dates
        dates.date_slider(date='2020-01-01', days_to_add=7)
        # '2020-01-08'
    """
    return (datetime.strptime(date, '%Y-%m-%d') + timedelta(days=days_to_add)).strftime('%Y-%m-%d')


import datetime
import pandas as pd
import time

def is_business_day(date):
    """Tells us whether a given date is a business day or note

    Note:
        useful for when we want to see if trading is possible on some given day

    Args:
        date: the date to be query

    Returns:
        0 for weekends, 1 for weekdays

    **Example**

    .. code-block:: python

        from wsbtrading.utils import dates
        dates.is_business_day(date='2020-01-01', '2020-01-01)
        # '2020-01-08'
    """
    return bool(len(pd.bdate_range(date, date)))

    date.isin(holidays)

import pandas as pd
from pandas.tseries.holiday import USFederalHolidayCalendar as calendar

date='2020-01-01'
cal = calendar()
holidays = cal.holidays(start=date, end=date)
date == holidays

is_business_day(date='2020-01-01')


"""Functions to aid in quick maths to calculate for ad-hoc analysis and feature engineering."""
from typing import Optional
import pandas as pd

from wsbtrading import check_columns


def divide_kernel(numerator: float, denominator: float) -> float:
    """Divides one number by another number.

    Args:
        numerator: the number for the top of your fraction
        denominator: the number for the bottom of your fraction

    Returns:
        a float representing one number that was divided by another number

    **Example**

    .. code-block:: python

        from wsbtrading.maths
        maths.divide_kernel(numerator=3, denominator=9)
    """
    if denominator == 0:
        return 0
    return numerator / denominator


def divide(df: 'DataFrame', numerator_col: str, denominator_col: str) -> pd.DataFrame:
    """Divides one number by another.

    Args:
        df: the dataframe to append a column onto
        numerator_col: the name of your numerator column
        denominator_col: the name of your denominator column

    Returns:
        the original dataframe with the division result appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.divide(df=df)
    """
    df = df.copy()
    check_columns(dataframe=df, required_columns=[numerator_col, denominator_col])

    df[f'{numerator_col}_perc_{denominator_col}'] = df[numerator_col] / df[denominator_col]
    return df


def sma(df: 'Dataframe', metric_col: str, rolling_window: Optional[int] = 20) -> pd.DataFrame:
    """Calculates the simple moving average (SMA) over a given time window.

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the moving average appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.sma(df=df, metric_col='Close', rolling_window=20)
    """
    df = df.copy()
    rolling_window_string = str(rolling_window)

    df[f'{rolling_window_string}sma'] = df[metric_col].rolling(window=rolling_window).mean()
    return df


def ema(df: 'Dataframe', metric_col: str, rolling_window: Optional[int] = 20) -> pd.DataFrame:
    """Calculates the exponential moving average (EMA) over a given time window.
    For more on ema versus sma, please [see this article](https://www.investopedia.com/ask/answers/122314/what-exponential-moving-average-ema-formula-and-how-ema-calculated.asp)

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the exponential moving average appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.ema(df=df, metric_col='Close', rolling_window=20)
    """
    df = df.copy()
    rolling_window_string = str(rolling_window)

    df[f'{rolling_window_string}ema'] = df[metric_col].rolling(window=rolling_window).mean()
    return df


def rolling_stddev(df: 'Dataframe', metric_col: str, rolling_window: Optional[int] = 20) -> pd.DataFrame:
    """Calculates the moving standard deviation over a given time window.

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the moving standard deviation appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.rolling_stddev(df=df, metric_col='Close', rolling_window=20)
    """
    df = df.copy()
    rolling_window_string = str(rolling_window)

    df[f'{rolling_window_string}stddev'] = df[metric_col].rolling(window=rolling_window).std()
    return df


def lower_band(df: 'Dataframe', metric_col: str, rolling_window: Optional[int] = 20) -> pd.DataFrame:
    """Calculates the lower bound of a stock's price movements.

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the lower bound appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.lower_band(df=df, metric_col='Close')
    """
    rolling_window_string = str(rolling_window)

    df = sma(df=df, metric_col=metric_col, rolling_window=rolling_window)
    df = rolling_stddev(df=df, metric_col=metric_col, rolling_window=rolling_window)

    df['lower_band'] = df[f'{rolling_window_string}sma'] - (2 * df[f'{rolling_window_string}stddev'])
    return df


def upper_band(df: 'Dataframe', metric_col: str, rolling_window: Optional[int] = 20) -> pd.DataFrame:
    """Calculates the lower bound of a stock's price movements.

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the lower bound appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.upper_band(df=df, metric_col='Close')
    """
    rolling_window_string = str(rolling_window)

    df = sma(df=df, metric_col=metric_col, rolling_window=rolling_window)
    df = rolling_stddev(df=df, metric_col=metric_col, rolling_window=rolling_window)

    df['upper_band'] = df[f'{rolling_window_string}sma'] + (2 * df[f'{rolling_window_string}stddev'])
    return df


def true_range(df: 'Dataframe', low_col: str, high_col: str) -> pd.DataFrame:
    """Calculates the true range (TR) for a stocks price movement.

    Args:
        df: the dataframe to append a column onto
        low_col: the column with the low price
        high_col: the column with the high price

    Returns:
        the original dataframe with the true range appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.true_range(df=df, low_col='Low', high_col='High')
    """
    df = df.copy()
    df['true_range'] = abs(df[high_col] - df[low_col])
    return df


def avg_true_range(df: 'Dataframe', low_col: str, high_col: str, rolling_window: Optional[int] = 20) -> pd.DataFrame:
    """Calculates the true range (TR) for a stocks price movement over a given time window.

    Args:
        df: the dataframe to append a column onto
        low_col: the column with the low price
        high_col: the column with the high price
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the true range appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.avg_true_range(df=df, low_col='Low', high_col='High', rolling_window=20)
    """
    df = df.copy()
    df = true_range(df=df, low_col=low_col, high_col=high_col)

    df['ATR'] = df['true_range'].rolling(window=rolling_window).mean()
    return df


def lower_keltner(df: 'Dataframe', metric_col: str, low_col: str, high_col: str, rolling_window: Optional[int] = 20) \
        -> pd.DataFrame:
    """Calculates the lower Keltner of a stock's price movements.

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        low_col: the column with the low price
        high_col: the column with the high price
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the lower bound appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.lower_keltner(df=df, metric_col='Close', low_col='Low', high_col='High', rolling_window=20)
    """
    rolling_window_string = str(rolling_window)

    df = sma(df=df, metric_col=metric_col, rolling_window=rolling_window)
    df = avg_true_range(df=df, low_col=low_col, high_col=high_col, rolling_window=rolling_window)

    df['lower_keltner'] = df[f'{rolling_window_string}sma'] - (df['ATR'] * 1.5)
    return df


def upper_keltner(df: 'Dataframe', metric_col: str, low_col: str, high_col: str, rolling_window: Optional[int] = 20) \
        -> pd.DataFrame:
    """Calculates the upper Keltner of a stock's price movements.

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        low_col: the column with the low price
        high_col: the column with the high price
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the lower bound appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.upper_keltner(df=df, metric_col='Close', low_col='Low', high_col='High', rolling_window=20)
    """
    rolling_window_string = str(rolling_window)

    df = sma(df=df, metric_col=metric_col, rolling_window=rolling_window)
    df = avg_true_range(df=df, low_col=low_col, high_col=high_col, rolling_window=rolling_window)

    df['upper_keltner'] = df[f'{rolling_window_string}sma'] + (df['ATR'] * 1.5)
    return df


def is_in_squeeze(df: 'Dataframe', metric_col: str, low_col: str, high_col: str, look_back_period: Optional[int] = -3,
                  rolling_window: Optional[int] = 20) -> bool:
    """Calculates whether a stock's price moments are indicative of an upcoming squeeze (i.e. going to the moon!🚀🚀🚀).

    Args:
        df: the dataframe to append a column onto
        metric_col: the column to calculate over (usually the 'Close' price)
        low_col: the column with the low price
        high_col: the column with the high price
        look_back_period: the number of days to look back
        rolling_window: the time window to calculate over

    Returns:
        the original dataframe with the lower bound appended

    **Example**

    .. code-block:: python

        from wsbtrading import maths
        df_mapped = maths.is_in_squeeze(
            df=df,
            metric_col='Close',
            low_col='Low',
            high_col='High',
            rolling_window=20
        )
    """
    lower_band_df = lower_band(df=df, metric_col=metric_col)
    upper_band_df = upper_band(df=lower_band_df, metric_col=metric_col)
    lower_keltner_df = lower_keltner(df=upper_band_df, metric_col=metric_col, low_col=low_col, high_col=high_col,
                                     rolling_window=rolling_window)
    upper_keltner_df = upper_keltner(df=lower_keltner_df, metric_col=metric_col, low_col=low_col, high_col=high_col,
                                     rolling_window=rolling_window)

    return upper_keltner_df['lower_band'].iloc[look_back_period] \
           > upper_keltner_df['lower_keltner'].iloc[look_back_period] \
           and upper_keltner_df['upper_band'].iloc[look_back_period] \
           < upper_keltner_df['upper_keltner'].iloc[look_back_period]



import unittest
import math
import pandas as pd
from pandas._testing import assert_frame_equal

from wsbtrading import maths


class TestDivision(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['timestamp', 'low', 'high', 'low_perc_high']
        data = [
            (1, 1, 1, 1),
            (2, 1, 2, 0.5),
            (3, 3, 9, 0.3333333333333333),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_divide_kernel(self):
        """Ensure we can divide properly."""
        actual = maths.divide_kernel(numerator=1, denominator=1)
        assert actual == 1

        actual = maths.divide_kernel(numerator=3, denominator=9)
        assert math.isclose(actual, 0.33333333333)

        actual = maths.divide_kernel(numerator=3, denominator=0)
        assert actual == 0

    def test_divide(self):
        """Ensures we can correctly divide when using a pandas DF."""
        mock_df = self.expected_df.drop('low_perc_high', axis=1)

        actual = maths.divide(df=mock_df, numerator_col='low', denominator_col='high')

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestSma(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', '2sma']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 30, 25.0),
            ('2017-01-05', 42, 32, 40, 35.0),
            ('2017-01-06', 52, 45, 50, 45.0),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_sma(self):
        """Ensures we correctly calculate the simple moving average (SMA)."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.sma(df=mock_df, metric_col='Close', rolling_window=2)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestStandardDeviation(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', '2stddev']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 7.778175),
            ('2017-01-05', 42, 32, 40, 6.363961),
            ('2017-01-06', 52, 45, 51, 7.778175),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_rolling_stddev(self):
        """Ensures we correctly calculate the simple moving average (SMA)."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.rolling_stddev(df=mock_df, metric_col='Close', rolling_window=2)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestLowerBand(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'lower_band']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 9.943651),
            ('2017-01-05', 42, 32, 40, 22.772078),
            ('2017-01-06', 52, 45, 51, 29.943651),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_lower_band(self):
        """Ensures we correctly calculate the lower band."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.lower_band(df=mock_df, metric_col='Close', rolling_window=2).drop(['2sma', '2stddev'], axis=1)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestUpperBand(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'upper_band']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 41.056349),
            ('2017-01-05', 42, 32, 40, 48.227922),
            ('2017-01-06', 52, 45, 51, 61.056349),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_upper_band(self):
        """Ensures we correctly calculate the upper band."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.upper_band(df=mock_df, metric_col='Close', rolling_window=2).drop(['2sma', '2stddev'], axis=1)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestTrueRange(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'true_range']
        data = [
            ('2017-01-03', 22, 20, 20, 2),
            ('2017-01-04', 32, 20, 31, 12),
            ('2017-01-05', 42, 32, 40, 10),
            ('2017-01-06', 52, 45, 51, 7),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_true_range(self):
        """Ensures we correctly calculate the true range."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.true_range(df=mock_df, low_col='Low', high_col='High')

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestAtr(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'ATR']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 7.0),
            ('2017-01-05', 42, 32, 40, 11.0),
            ('2017-01-06', 52, 45, 51, 8.5),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_avg_true_range(self):
        """Ensures we correctly calculate the average true range."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.avg_true_range(df=mock_df, low_col='Low', high_col='High', rolling_window=2)\
            .drop(['true_range'], axis=1)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestLowerKeltner(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'lower_keltner']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 15.00),
            ('2017-01-05', 42, 32, 40, 19.00),
            ('2017-01-06', 52, 45, 51, 32.75),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_lower_keltner(self):
        """Ensures we correctly calculate the lower Keltner."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.lower_keltner(df=mock_df, metric_col='Close', low_col='Low', high_col='High', rolling_window=2) \
            .drop(['2sma', 'true_range', 'ATR'], axis=1)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestUpperKeltner(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'upper_keltner']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 36.00),
            ('2017-01-05', 42, 32, 40, 52.00),
            ('2017-01-06', 52, 45, 51, 58.25),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_upper_keltner(self):
        """Ensures we correctly calculate the upper Keltner."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.upper_keltner(df=mock_df, metric_col='Close', low_col='Low', high_col='High', rolling_window=2) \
            .drop(['2sma', 'true_range', 'ATR'], axis=1)

        assert_frame_equal(actual, self.expected_df, check_dtype=True)


class TestIsInSqueeze(unittest.TestCase):
    def setUp(self) -> None:
        # yapf: disable
        schema = ['Date', 'High', 'Low', 'Close', 'upper_keltner']
        data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 36.00),
            ('2017-01-05', 42, 32, 40, 52.00),
            ('2017-01-06', 52, 45, 51, 58.25),
        ]
        # yapf: enable
        self.expected_df = pd.DataFrame(data=data, columns=schema)

    def test_is_in_squeeze(self):
        """Ensures we correctly calculate when a stock is squeezing."""
        mock_df = self.expected_df[['Date', 'High', 'Low', 'Close']]

        actual = maths.is_in_squeeze(df=mock_df, metric_col='Close', low_col='Low', high_col='High', rolling_window=2)

        self.assertFalse(actual)



import math
import pandas as pd
from pandas._testing import assert_frame_equal

from wsbtrading import maths

schema = ['Date', 'High', 'Low', 'Close', 'lower_keltner']
data = [
            ('2017-01-03', 22, 20, 20, None),
            ('2017-01-04', 32, 20, 31, 15.00),
            ('2017-01-05', 42, 32, 40, 19.00),
            ('2017-01-06', 52, 45, 51, 32.75),
        ]
# yapf: enable
expected_df = pd.DataFrame(data=data, columns=schema)

mock_df = expected_df[['Date', 'High', 'Low', 'Close']]

metric_col='Close'
low_col='Low'
high_col='High'
rolling_window=2
lower_band_df = maths.lower_band(df=mock_df, metric_col=metric_col, rolling_window=rolling_window)
upper_band_df = maths.upper_band(df=lower_band_df, metric_col=metric_col, rolling_window=rolling_window)
lower_keltner_df = maths.lower_keltner(df=upper_band_df, metric_col=metric_col, low_col=low_col, high_col=high_col,
                                 rolling_window=rolling_window)
upper_keltner_df = maths.upper_keltner(df=lower_keltner_df, metric_col=metric_col, low_col=low_col, high_col=high_col,
                                 rolling_window=rolling_window)

def is_squeeze(df):
    return df['lower_band'].iloc[-3] > df['lower_keltner'].iloc[-3] and df['upper_band'].iloc[-3] < df['upper_keltner'].iloc[-3]

is_squeeze(df=upper_keltner_df)

actual = maths.is_in_squeeze(df=mock_df, metric_col='Close', low_col='Low', high_col='High', rolling_window=2)
actual

assert actual == False





# --------
# Alpaca |
# --------
class Alpaca:
    api_key = 'PK2TECTJURINF9NGBQT8'
    secret_key = 'q4JCbfISvU3Gq6sLY2dnAr95fs1Ljnut8z3peNw1'
    urls = {
        'base_url': 'https://paper-api.alpaca.markets',
        'account_url': f'{BASE_URL}/v2/account',
        'actionContext': f'{BASE_URL}/v2/order',
    }
    headers = {
        'APCA-API-KEY-ID': api_key,
        'APCA-API-SECRET-KEY': secret_key,
    }




import os
from datetime import datetime
from typing import Optional, List
from abc import abstractmethod
import csv
import psycopg2
import quandl
import numpy as np
import pandas as pd

import alpaca_trade_api as tradeapi

from wsbtrading.instrumentation import Alpaca as iAlpaca
from wsbtrading.instrumentation import Quandl as iQuandl


class CSV:
    @abstractmethod
    def writerow(self, row: List[str]) -> None:
        pass

    @abstractmethod
    def writerows(self, rows: List[List[str]]) -> None:
        pass

    @abstractmethod
    def dialect(self) -> csv.Dialect:
        pass


def quandl_api():
    """Creates the initial connection to Quandl API.

    Args:
        subscription: denotes subscription API to connect to

    Returns:
        an API connection to Alpaca

    **Example**

    .. code-block:: python

        import quandl
        from wsbtrading import data_io

        data_io.quandl_api()
        quandl.get_table('SHARADAR/SEP', date='2013-03-15', ticker='ZZ')
    """
    quandl.ApiConfig.api_key = iQuandl.api_key


def alpaca_rest_api_conn(trading_type: str):
    """Creates the initial connection to Alpaca API.

    Args:
        trading_type: denotes live versus paper trading

    Returns:
        an API connection to Alpaca

    **Example**

    .. code-block:: python

        from wsbtrading import data_io
        alpaca_api = data_io.alpaca_rest_api_conn(trading_type='paper_trading')
    """
    api_key = iAlpaca.api_key
    secret_key = iAlpaca.secret_key
    base_url = iAlpaca.api_call[trading_type]['base_url']

    return tradeapi.REST(api_key, secret_key, base_url=base_url)


def postgres_conn(database: str = 'wsbtrading',
            user: str = 'postgres',
            password: str = 'postgres',
            host: str = '127.0.0.1',
            port: str = '5432'):
    """Creates the initial connection to Postgres.

    Args:
        database: denotes live versus paper trading
        user: the user to sign in as
        password: the password to sign in with
        host: the host URL
        port: the port number

    Returns:
        a Postgres connection

    **Example**

    .. code-block:: python

        from wsbtrading import data_io
        postgres_conn = data_io.postgres_conn()
    """
    conn = psycopg2.connect(database=database, user=user, password=password, host=host, port=port)
    cur = conn.cursor()

    return conn, cur


def generate_path_to_write(environment: str,
                           granularity: str,
                           dataset_name: str,
                           root_path: str = '/group/wsbtrading',
                           timestamp: Optional[str] = None):
    """Generates an HDFS path from the inputs.
    The output path is of the form ``/ROOT_PATH/ENVIRONMENT/GRANULARITY/DATASET_NAME/TIMESTAMP``

    Args:
        environment: possible values are 'dev' or 'prod'
        granularity: possible values are 'stream', 'minute', 'daily', 'weekly', 'monthly'
        dataset_name: the name of the dataset, with no spaces e.g. stock_ticker_metadata
        root_path: the root for your dataset path, e.g. ``/user/your_user`` or ``/group/wsbtrading``.
        Defaults to ``/group/wsbtrading``.
        experiment_id: the unique identifer for the experiment
        timestamp: the timestamp in UTC when the job ran, autogenerated. Can be overridden e.g. if user wants to
                   pass in some id like 'v1' or 'run10' or 'run_with_fixes'.

    Returns:
        The path to be written to

    **Example**
    .. code-block:: python

        from phobos import data_io
        data_io.generate_path_to_write(
            environment='prod',
            granularity='daily',
            dataset_name='stock_ticker_metadata'
        )
        # /group/wsbtrading/prod/aggregate/stock_ticker_metadata/file1.parquet...file2.parquet
    """
    supported_environments = ['dev', 'prod']
    supported_granularities = ['stream', 'minute', 'daily', 'weekly', 'monthly']

    if environment not in supported_environments or granularity not in supported_granularities:
        raise ValueError('Unsupported environment or granularity.')

    dataset_name = dataset_name.replace(' ', '_').lower()
    timestamp = timestamp or datetime.utcnow().strftime('%Y-%m-%d_%H-%M-%S')

    return os.path.join(root_path, environment, granularity, dataset_name, timestamp)


def _get_table_field_names(table_name: str) -> List[str]:
    """Queries the field names of a given table.

    Args:
        table_name: the name of the table to query

    Returns:
        a list of column names

    **Example**

    .. code-block:: python

        from phobos import data_io
        column_list = data_io._get_table_field_names(table_name='company_listing_status')
    """
    try:
        cur.execute(f'Select * FROM {table_name} LIMIT 0')
        col_names = [desc[0] for desc in cur.description]
    except psycopg2.Error as e:
        print(f'error: {e}')

    return col_names


def insert_csv_to_sql(table_name: str, csv_path: str, delimiter: str = ','):
    """Given a SQL table.

    Args:
        table_name: the name of the table to query
        csv_path: the path to the data file to import
        delimiter: the type of delimiter for the file

    **Example**

    .. code-block:: python

        from phobos import data_io
        data_io.insert_csv_to_sql(table_name='company_listing_status', csv_path='../file.csv')
    """
    col_names = _get_table_field_names(table_name=table_name)

    try:
        df = pd.read_csv(csv_path, index_col=0)
        df.replace({np.nan: None})
        df.to_csv('clean_csv.csv')
        csv_contents = open('clean_csv.csv', 'r')
        next(csv_contents)
    except psycopg2.Error as e:
        print(f'error: {e}')

    try:
        cur.copy_from(file=csv_contents,
                      table=table_name,
                      null=None,
                      columns=col_names,
                      sep=delimiter)
        conn.commit()
        cur.close()
        conn.close()
        print("Success!")
    except psycopg2.Error as e:
        print(f'error: {e}')






# https://docs.databricks.com/spark/latest/spark-sql/spark-pandas.html
# Enable Arrow-based columnar data transfers
# spark.conf.set("spark.sql.execution.arrow.pyspark.enabled", "true")
#
# # Generate a pandas DataFrame
# pdf = pd.DataFrame(np.random.rand(100, 3))
#
# # Create a Spark DataFrame from a pandas DataFrame using Arrow
# df = spark.createDataFrame(pdf)
#
# # Convert the Spark DataFrame back to a pandas DataFrame using Arrow
# result_pdf = df.select("*").toPandas()

# import config
# import websocket
# import json
# import requests

# def on_open(ws):
#     print("opened")
#     auth_data = {
#         "action": "authenticate",
#         "data": {"key_id": config.API_KEY, "secret_key": config.SECRET_KEY}
#     }
#
#     ws.send(json.dumps(auth_data))
#
#     listen_message = {"action": "listen", "data": {"streams": ["AM.TSLA"]}}
#
#     ws.send(json.dumps(listen_message))
#
#
# def on_message(ws, message):
#     print("received a message")
#     print(message)
#
#
# def on_close(ws):
#     print("closed connection")
#
#
# socket = "wss://data.alpaca.markets/stream"
#
#
# ws = websocket.WebSocketApp(socket, on_open=on_open, on_message=on_message, on_close=on_close)
# ws.run_forever()



#!/usr/bin/env python
import pathlib
import subprocess
import sys

from setuptools import setup, find_packages

import wsbtrading

with open('README.md', 'r', encoding='utf-8') as readme:
    DESCRIPTION = readme.read()

version = wsbtrading.__version__

for arg in sys.argv:
    if arg.startswith(wsbtrading.__version__):
        staging_version = sys.argv[2]
        version = staging_version
        sys.argv.remove(staging_version)
        break

def _get_sha() -> str:
    """Takes the sha from the HEAD commit."""
    project_root = pathlib.Path(__file__).parent
    try:
        return subprocess.check_output(['git', 'rev-parse', 'HEAD'], cwd=project_root).decode('ascii').strip()

    except subprocess.CalledProcessError:  # most likely runs from tox
        pass

for arg in sys.argv:
    if arg == 'dev-build':
        sha = _get_sha()
        if sha is not None:
            _hash = _get_sha()[:7]
            version += '+' + _hash
        sys.argv.remove(arg)
        break

DISTNAME = 'wsbtrading'
DESCRIPTION = """wsbtrading is a library that handles data I/O, aggregation,
and modeling to facilitate algorithmic trading stategies."""
MAINTAINER = 'bordumb'
# MAINTAINER_EMAIL = 'DM me here or on Reddit: @bordumb'
AUTHOR = 'bordumb'
AUTHOR_EMAIL = 'DM me here or on Reddit: @bordumb'
URL = "https://github.com/bordumb/wsbtrading"
LICENSE = "Apache License, Version 2.0"

classifiers = ['Programming Language :: Python',
               'Programming Language :: Python :: 3.6',
               'Intended Audience :: Science/Research',
               'Topic :: Scientific/Engineering',
               'Topic :: Scientific/Engineering :: Mathematics',
               'Operating System :: OS Independent',
               'Intended Audience :: Financial and Insurance Industry',
               'Intended Audience :: Information Technology']

if __name__ == "__main__":
    setup(
        name=DISTNAME,
        version=version,
        maintainer=MAINTAINER,
        # maintainer_email=MAINTAINER_EMAIL,
        description='wsbtrading is a library that handles data I/O, aggregation, and modeling to facilitate algorithmic trading stategies.',
        long_description=DESCRIPTION,
        license=LICENSE,
        url=URL,
        packages=find_packages(),
        classifiers=classifiers,
        test_suite='nose.collector',
    )



import logging
from time import sleep

import alpaca_trade_api as tradeapi
import pandas as pd

from wsbtrading.instrumentation import Alpaca as iAlpaca


def api_start():
    api_key = iAlpaca.api_key
    api_secret = iAlpaca.api_secret
    base_url = iAlpaca.live_trading_url
    api = tradeapi.REST(api_key, api_secret, base_url, api_version='v2')

    return api


def time_to_market_close() -> float:
    """Calculate the time (seconds) until the market closes

    Note:
        this is mostly to be used to stop ourselves from making orders to soon before the bell closes.

    **Example**

    .. code-block:: python

        from wsbtrading.order import utils
        utils.time_to_market_close()
    """
    api = api_start()
    clock = api.get_clock()

    return (clock.next_close - clock.timestamp).total_seconds()


def wait_for_market_open():
    """Function to make ourselves wait for the market to open.

    **Example**

    .. code-block:: python

        from wsbtrading.order import utils
        utils.time_to_market_close()
    """
    api = api_start()
    clock = api.get_clock()
    if not clock.is_open:
        time_to_open = (clock.next_open - clock.timestamp).total_seconds()
        sleep(round(time_to_open))



# --------
# Alpaca |
# --------
class Alpaca:
    # TODO: Turn these keys into environment variables for security (fine for now as it's paper wsbtrading)
    api_key = 'PK847B96RX8C9GJ3AMO7'
    secret_key = '3ZdZzKEXYwJQxQtq7JNK0KDeatZXzvQSvAwZPuqh'

    headers = {
        'APCA-API-KEY-ID': api_key,
        'APCA-API-SECRET-KEY': secret_key,
    }

    live_trading_url = 'https://api.alpaca.markets'
    paper_trading_url = 'https://paper-api.alpaca.markets'

    api_call = {
        'live_trading': {
            'base_url': 'https://api.alpaca.markets',
            'sub_urls': {
                'account_url': f'{live_trading_url}/v2/account',
                'order_url': f'{live_trading_url}/v2/orders',
            }
        },
        'paper_trading': {
            'base_url': 'https://paper-api.alpaca.markets',
            'sub_urls': {
                'account_url': f'{paper_trading_url}/v2/account',
                'order_url': f'{paper_trading_url}/v2/orders',
            }
        }
    }


# -----------------
# Alpha Advantage |
# https://www.alphavantage.co/documentation/#listing-status
# -----------------
class AlphaAdvantage:
    # TODO: Turn these keys into environment variables for security (fine for now as it's paper wsbtrading)
    api_key = 'XQ6PZ3J5YFU4VRIS'


# --------
# Quandl |
# --------
class Quandl:
    # TODO: Turn these keys into environment variables for security (fine for now as it's paper wsbtrading)
    api_key = 'xqq66bfL2zy6ijycPEWd'

